.module/RAM/ABS=0       DTMF;

.include    <..\system.k>;

{******************************************************************************
 *
 *  Variables
 *
 ******************************************************************************}

.VAR/DM    StateFlag;

{ dynamic scratchpad variables }
.var    hertz1, hertz2,             { row and col frequency in Hertz }
    sum1, sum2,                 { row and col phase accumulators }
    sin1, sin2,                 { returned values from calling sin }
    time_on,                    { software counter to manage tone ON time }
    time_off,                   { software counter to manage tone OFF time }
    state;                      { dialing demo state machine }

{ fixed variables to be loaded from booted PM }
.var    scale,                      { attenuation of each sine before summing }
    sign_dura_ms,               { signal duration time in millisec }
    interdigit_ms,              { interdigit time in millisec }
    digits[32],                 { lookup table for digit row,col freqs }
    dial_list[100];             { store dial sequence here }
{------------------------------------------------------------------------------+
|   store values in dial_list as follows:                                      |
|                                                               (E=*, F=#)     |
|   DTMF digits    ==> h#000y, where y=0,1,2,3,4,5,6,7,8,9,A,B,C,D,E,F         |
|   pause          ==> h#00yX, where y is non-zero, X is don't care            |
|   redial         ==> h#0yXX, where y is non-zero, X is don't care            |
|   stop           ==> h#yXXX, where y is non-zero, X is don't care            |
+------------------------------------------------------------------------------}

.init      scale:          h#FFFF;
.init      sign_dura_ms:   h#0064;     {  100 ms (in hex) }
.init      interdigit_ms:  h#0064;     {  100 ms (in hex) }
.init      digits[00]: h#03AD,h#0538, h#02B9,h#04B9, h#02B9,h#0538, h#02B9,h#05C5;
.init      digits[08]: h#0302,h#04B9, h#0302,h#0538, h#0302,h#05C5, h#0354,h#04B9;
.init      digits[16]: h#0354,h#0538, h#0354,h#05C5, h#02B9,h#0661, h#0302,h#0661;
.init      digits[24]: h#0354,h#0661, h#03AD,h#0661, h#03AD,h#04B9, h#03AD,h#05C5;
.init      dial_list[0]:   h#0001;     { dials author: 1-617-461-3058}
.init      dial_list[1]:   h#00ff;
.init      dial_list[2]:   h#0006;
.init      dial_list[3]:   h#0001;
.init      dial_list[4]:   h#0007;
.init      dial_list[5]:   h#00ff;
.init      dial_list[6]:   h#0004;
.init      dial_list[7]:   h#0006;
.init      dial_list[8]:   h#0001;
.init      dial_list[9]:   h#0003;
.init      dial_list[10]:  h#0000;
.init      dial_list[11]:  h#0005;
.init      dial_list[12]:  h#0008;
.init      dial_list[13]:  h#00ff;
.init      dial_list[14]:  h#00ff;      { now do the whole keypad }
.init      dial_list[15]:  h#0001;
.init      dial_list[16]:  h#0002;
.init      dial_list[17]:  h#0003;
.init      dial_list[18]:  h#000a;
.init      dial_list[19]:  h#00ff;
.init      dial_list[20]:  h#0004;
.init      dial_list[21]:  h#0005;
.init      dial_list[22]:  h#0006;
.init      dial_list[23]:  h#000b;
.init      dial_list[24]:  h#00ff;
.init      dial_list[25]:  h#0007;
.init      dial_list[26]:  h#0008;
.init      dial_list[27]:  h#0009;
.init      dial_list[28]:  h#000c;
.init      dial_list[29]:  h#00ff;
.init      dial_list[30]:  h#000e;
.init      dial_list[31]:  h#0000;
.init      dial_list[32]:  h#000f;
.init      dial_list[33]:  h#000d;
.init      dial_list[34]:  h#ffff;

.var/dm/ram/circ                rx_buf[3];      /* Status + L data + R data */
.var/dm/ram/circ                tx_buf[3];      /* Cmd + L data + R data    */
.var/dm/ram/circ                init_cmds[13];
.var/dm                         stat_flag;

.init tx_buf:   0xc000, 0x0000, 0x0000; /* Initially set MCE        */

.init init_cmds:
        0xc002,     {
                        Left input control reg
                        b7-6: 0=left line 1
                              1=left aux 1
                              2=left line 2
                              3=left line 1 post-mixed loopback
                        b5-4: res
                        b3-0: left input gain x 1.5 dB
                    }
        0xc102,     {
                        Right input control reg
                        b7-6: 0=right line 1
                              1=right aux 1
                              2=right line 2
                              3=right line 1 post-mixed loopback
                        b5-4: res
                        b3-0: right input gain x 1.5 dB
                    }
        0xc288,     {
                        left aux 1 control reg
                        b7  : 1=left aux 1 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc388,     {
                        right aux 1 control reg
                        b7  : 1=right aux 1 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc488,     {
                        left aux 2 control reg
                        b7  : 1=left aux 2 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc588,     {
                        right aux 2 control reg
                        b7  : 1=right aux 2 mute
                        b6-5: res
                        b4-0: gain/atten x 1.5, 08= 0dB, 00= 12dB
                    }
        0xc680,     {
                        left DAC control reg
                        b7  : 1=left DAC mute
                        b6  : res
                        b5-0: attenuation x 1.5 dB
                    }
        0xc780,     {
                        right DAC control reg
                        b7  : 1=right DAC mute
                        b6  : res
                        b5-0: attenuation x 1.5 dB
                    }
        0xc850,     {
                        data format register
                        b7  : res
                        b5-6: 0=8-bit unsigned linear PCM
                              1=8-bit u-law companded
                              2=16-bit signed linear PCM
                              3=8-bit A-law companded
                        b4  : 0=mono, 1=stereo
                        b0-3: 0=  8.
                              1=  5.5125
                              2= 16.
                              3= 11.025
                              4= 27.42857
                              5= 18.9
                              6= 32.
                              7= 22.05
                              8=   .
                              9= 37.8
                              a=   .
                              b= 44.1
                              c= 48.
                              d= 33.075
                              e=  9.6
                              f=  6.615
                       (b0) : 0=XTAL1 24.576 MHz; 1=XTAL2 16.9344 MHz
                    }
        0xc909,     {
                        interface configuration reg
                        b7-4: res
                        b3  : 1=autocalibrate
                        b2-1: res
                        b0  : 1=playback enabled
                    }
        0xca00,     {
                        pin control reg
                        b7  : logic state of pin XCTL1
                        b6  : logic state of pin XCTL0
                        b5  : master - 1=tri-state CLKOUT
                              slave  - x=tri-state CLKOUT
                        b4-0: res
                    }
        0xcc40,     {
                        miscellaneous information reg
                        b7  : 1=16 slots per frame, 0=32 slots per frame
                        b6  : 1=2-wire system, 0=1-wire system
                        b5-0: res
                    }
        0xcd00;     {
                        digital mix control reg
                        b7-2: attenuation x 1.5 dB
                        b1  : res
                        b0  : 1=digital mix enabled
                    }

{******************************************************************************
 *
 *  Interrupt vector table
 *
 ******************************************************************************}
        jump start;  rti; rti; rti;     {00: reset }
        rti;         rti; rti; rti;     {04: IRQ2 }
        rti;         rti; rti; rti;     {08: IRQL1 }
        rti;         rti; rti; rti;     {0c: IRQL0 }
        ar = dm(stat_flag);             {10: SPORT0 tx }
        ar = pass ar;
        if eq rti;
        jump next_cmd;
        jump input_samples;             {14: SPORT1 rx }
                     rti; rti; rti;
        rti;         rti; rti; rti;     {18: IRQE }
        rti;         rti; rti; rti;     {1c: BDMA }
        jump irq1isr;
                     rti; rti; rti;     {20: SPORT1 tx or IRQ1 }
        rti;         rti; rti; rti;     {24: SPORT1 rx or IRQ0 }
        rti;         rti; rti; rti;     {28: timer }
        rti;         rti; rti; rti;     {2c: power down }


{******************************************************************************
 *
 *  ADSP 2181 intialization
 *
 ******************************************************************************}
start:
        {   shut down sport 0 }
        ax0 = b#0000100000000000;   dm (System_Control_Reg) = ax0;

{   restores monitor timer handler. }
        i7 = 0x3fe8;
        ar = pm (i7, m7);               { px implicit }
        i7 = 0x28;
        pm (i7, m7) = ar;


        i5 = ^rx_buf;
        l5 = %rx_buf;
        i6 = ^tx_buf;
        l6 = %tx_buf;
        i3 = ^init_cmds;
        l3 = %init_cmds;

        m1 = 1;
        m5 = 1;


{================== S E R I A L   P O R T   #0   S T U F F ==================}
        ax0 = b#0000110011010111;   dm (SPORT0_Autobuf) = ax0;
            {   |||!|-/!/|-/|/|+- receive autobuffering 0=off, 1=on
                |||!|  ! |  | +-- transmit autobuffering 0=off, 1=on
                |||!|  ! |  +---- | receive m?
                |||!|  ! |        | m5
                |||!|  ! +------- ! receive i?
                |||!|  !          ! i5
                |||!|  !          !
                |||!|  +========= | transmit m?
                |||!|             | m5
                |||!+------------ ! transmit i?
                |||!              ! i6
                |||!              !
                |||+============= | BIASRND MAC biased rounding control bit
                ||+-------------- 0
                |+--------------- | CLKODIS CLKOUT disable control bit
                +---------------- 0
            }

        ax0 = 0;    dm (SPORT0_RFSDIV) = ax0;
            {   RFSDIV = SCLK Hz/RFS Hz - 1 }
        ax0 = 0;    dm (SPORT0_SCLKDIV) = ax0;
            {   SCLK = CLKOUT / (2  (SCLKDIV + 1) }
        ax0 = b#1000011000001111;   dm (SPORT0_Control_Reg) = ax0;
            {   multichannel
                ||+--/|!||+/+---/ | number of bit per word - 1
                |||   |!|||       | = 15
                |||   |!|||       |
                |||   |!|||       |
                |||   |!||+====== ! 0=right just, 0-fill; 1=right just, signed
                |||   |!||        ! 2=compand u-law; 3=compand A-law
                |||   |!|+------- receive framing logic 0=pos, 1=neg
                |||   |!+-------- transmit data valid logic 0=pos, 1=neg
                |||   |+========= RFS 0=ext, 1=int
                |||   +---------- multichannel length 0=24, 1=32 words
                ||+-------------- | frame sync to occur this number of clock
                ||                | cycle before first bit
                ||                |
                ||                |
                |+--------------- ISCLK 0=ext, 1=int
                +---------------- multichannel 0=disable, 1=enable
            }
            {   non-multichannel
                |||!|||!|||!+---/ | number of bit per word - 1
                |||!|||!|||!      | = 15
                |||!|||!|||!      |
                |||!|||!|||!      |
                |||!|||!|||+===== ! 0=right just, 0-fill; 1=right just, signed
                |||!|||!||+------ ! 2=compand u-law; 3=compand A-law
                |||!|||!|+------- receive framing logic 0=pos, 1=neg
                |||!|||!+-------- transmit framing logic 0=pos, 1=neg
                |||!|||+========= RFS 0=ext, 1=int
                |||!||+---------- TFS 0=ext, 1=int
                |||!|+----------- TFS width 0=FS before data, 1=FS in sync
                |||!+------------ TFS 0=no, 1=required
                |||+============= RFS width 0=FS before data, 1=FS in sync
                ||+-------------- RFS 0=no, 1=required
                |+--------------- ISCLK 0=ext, 1=int
                +---------------- multichannel 0=disable, 1=enable
            }


        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels0) = ax0;
            {   ^15          00^   transmit word enables: channel # == bit # }
        ax0 = b#0000000000000111;   dm (SPORT0_TX_Channels1) = ax0;
            {   ^31          16^   transmit word enables: channel # == bit # }
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels0) = ax0;
            {   ^15          00^   receive word enables: channel # == bit # }
        ax0 = b#0000000000000111;   dm (SPORT0_RX_Channels1) = ax0;
            {   ^31          16^   receive word enables: channel # == bit # }


{============== S Y S T E M   A N D   M E M O R Y   S T U F F ==============}
        ax0 = b#0001100000000000;   dm (System_Control_Reg) = ax0;
            {   +-/!||+-----/+-/- | program memory wait states
                |  !|||           | 0
                |  !|||           |
                |  !||+---------- 0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !||            0
                |  !|+----------- SPORT1 1=serial port, 0=FI, FO, IRQ0, IRQ1,..
                |  !+------------ SPORT1 1=enabled, 0=disabled
                |  +============= SPORT0 1=enabled, 0=disabled
                +---------------- 0
                                  0
                                  0
            }



        ifc = b#00000011111111;         { clear pending interrupt }
        nop;


        icntl = b#00010;
            {     ||||+- | IRQ0: 0=level, 1=edge
                  |||+-- | IRQ1: 0=level, 1=edge
                  ||+--- | IRQ2: 0=level, 1=edge
                  |+---- 0
                  |----- | IRQ nesting: 0=disabled, 1=enabled
            }


        mstat = b#1000000;
            {     ||||||+- | Data register bank select
                  |||||+-- | FFT bit reverse mode (DAG1)
                  ||||+--- | ALU overflow latch mode, 1=sticky
                  |||+---- | AR saturation mode, 1=saturate, 0=wrap
                  ||+----- | MAC result, 0=fractional, 1=integer
                  |+------ | timer enable
                  +------- | GO MODE
            }





{******************************************************************************
 *
 *  ADSP 1847 Codec intialization
 *
 ******************************************************************************}

        {   clear flag }
        ax0 = 1;
        dm(stat_flag) = ax0;

        {   enable transmit interrupt }
        ena ints;
        imask = b#0001000000;
            {     |||||||||+ | timer
                  ||||||||+- | SPORT1 rec or IRQ0
                  |||||||+-- | SPORT1 trx or IRQ1
                  ||||||+--- | BDMA
                  |||||+---- | IRQE
                  ||||+----- | SPORT0 rec
                  |||+------ | SPORT0 trx
                  ||+------- | IRQL0
                  |+-------- | IRQL1
                  +--------- | IRQ2
            }


        ax0 = dm (i6, m5);          { start interrupt }
        tx0 = ax0;

check_init:
        ax0 = dm (stat_flag);       { wait for entire init }
        af = pass ax0;              { buffer to be sent to }
        if ne jump check_init;      { the codec            }

        ay0 = 2;
check_aci1:
        ax0 = dm (rx_buf);          { once initialized, wait for codec }
        ar = ax0 and ay0;           { to come out of autocalibration }
        if eq jump check_aci1;      { wait for bit set }

check_aci2:
        ax0 = dm (rx_buf);          { wait for bit clear }
        ar = ax0 and ay0;
        if ne jump check_aci2;
        idle;

        ay0 = 0xbf3f;               { unmute left DAC }
        ax0 = dm (init_cmds + 6);
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;

        ax0 = dm (init_cmds + 7);   { unmute right DAC }
        ar = ax0 AND ay0;
        dm (tx_buf) = ar;
        idle;


        ifc = b#00000011111111;     { clear any pending interrupt }
        nop;


{   end codec initialization, begin filter demo initialization }



        l0=0; l1=0; l2=0; l3=0; l4=0; l7=0;
        i0=^dial_list;
        m0=1;
        m7=1; l7=0;         { used by boot_sine routine }
        dm(state)=m0;       { start up in state 1 (i.e. dial tone) }

        imask = b#0000100101;       { enable rx0 interrupt }
            {     |||||||||+ | timer
                  ||||||||+- | SPORT1 rec or IRQ0
                  |||||||+-- | SPORT1 trx or IRQ1
                  ||||||+--- | BDMA
                  |||||+---- | IRQE
                  ||||+----- | SPORT0 rec
                  |||+------ | SPORT0 trx
                  ||+------- | IRQL0
                  |+-------- | IRQL1
                  +--------- | IRQ2
            }

        jump iSend;         {   send dummy to signal ready }

{------------------------------------------------------------------------------
 -
 -  command loop.
 -
 ------------------------------------------------------------------------------}
again:  { any thing from host ?}
        ar = dm (CHAR_WAITING_FLAG);            { kbhit () }
        none = pass ar;
        if ne jump again;

{ has something }
        i4 = dm (PTR_TO_GET_CHAR);        { get int }
        call (i4);
        if lt jump again; { time out }
        ay0 = 68;   { 'D' }
        none = ax1 - ay0;
        if eq jump dial;
        ay0 = 84;   { 'T' }
        none = ax1 - ay0;
        if eq jump tone;
        ay0 = 90;   { 'Z' }
        none = ax1 - ay0;
        if eq rts;          { go back to monitor. }
        ar = ax1 + 1;
        ax1 = ar;
iSend:  i4 = dm (PTR_TO_OUT_CHAR);        { send int }
        call (i4);

{ wait for char to go out }
wt:
        ar = dm (CHAR_SEND_DONE_FLAG);
        none = pass ar;
        if eq jump wt;

        toggle fl1;
        jump again;




dial:   { geterate phone numbers }
        i0=^dial_list;
        call newsound;
        ar = 2;
        dm(state)=ar;

        { return 'd' as ack. }
        ax1 = 100; { 'd' }
        jump iSend;


tone:   { generate dial tone }
        i0=^dial_list;
        call newsound;
        ar = 1;
        dm(state)=ar;

        { return 't' as ack. }
        ax1 = 116; { 't' }
        jump iSend;



{------------------------------------------------------------------------------}
newsound:   si=0;                       { clear phase accumulators }
        dm(sum1)=si;                { tone 1 }
        dm(sum2)=si;                { tone 2 }

        si=dm(sign_dura_ms);
        sr=ashift si by 3 (hi);     { time ms x 8 = no. of 8 kHz samples }
        ay0=sr1;
        ar=ay0-1;                   { count N-1 to 0, not N to 1 }
        dm(time_on)=ar;             { init signal timer (counts samples) }

        si=dm(interdigit_ms);
        sr=ashift si by 3 (hi);     { time ms x 8 = no. of 8 kHz samples }
        ay0=sr1;
        ar=ay0-1;                   { count N-1 to 0, not N to 1 }
        dm(time_off)=ar;            { init silence timer (counts samples) }
        rts;

{------------------------------------------------------------------------------
 -
 -  SPORT0 interrupt handler
 -
 ------------------------------------------------------------------------------}
input_samples:
        ena sec_reg;                { use shadow register bank }

        ay0=2;
        ax0=dm(state);      { determine "state" of state machine }
        ar=ax0-ay0;
        if eq jump state2;
        af=ay0-1;
        ar=ax0-af;
        if eq jump state1;

{ --- state 0 --- }
state0:     rti;

{ --- state 1 --- }
state1:     ax0=350;
        dm(hertz1)=ax0;
        ax0=440;
        dm(hertz2)=ax0;
        jump maketones;

{ --- state 2 --- }
state2:     ay0=dm(time_on);
        none=pass ay0;
        if eq jump quiet;       { signal? or silence? now }
        ar=ay0-1;
        dm(time_on)=ar;         { decr. sig duration counter }
maketones:  se=dm(scale);

tone1:      ay0=dm(sum1);
        si=dm(hertz1);          { freq stored as Hz in DM }
        sr=ashift si by 3 (hi);
        my0=h#4189;             { mult Hz by .512 * 2 }
        mr=sr1*my0(rnd);        { i.e. mult by 1.024 }
        sr=ashift mr1 by 1 (hi);
        ar=sr1+ay0;
        dm(sum1)=ar;
        ax0=ar;
        call boot_sin;
        sr=ashift ar (hi);      { scale value in SE }
        dm(sin1)=sr1;

tone2:      ay0=dm(sum2);
        si=dm(hertz2);          { freq stored as Hz in DM }
        sr=ashift si by 3 (hi);
        my0=h#4189;             { mult Hz by .512 * 2 }
        mr=sr1*my0(rnd);        { i.e. mult by 1.024 }
        sr=ashift mr1 by 1 (hi);
        ar=sr1+ay0;
        dm(sum2)=ar;
        ax0=ar;
        call boot_sin;
        sr=ashift ar (hi);      { scale value in SE }
        dm(sin2)=sr1;

add_em:     ax0=dm(sin1);
        ay0=dm(sin2);
        ar=ax0+ay0;

sound:      sr=ashift ar by -2 (hi);  { compand 14 LSBs only! }
        dm (tx_buf + 1) = sr1;        { write to output auto-buffer }
        dm (tx_buf + 2) = sr1;        { write to output auto-buffer }
        rti;

quiet:      ay0=dm(time_off);
        ar=pass ay0;
        if eq jump nextdigit;
        ar=ay0-1;
        dm(time_off)=ar;              { decr. silence duration counter }
        ar=0;
        dm (tx_buf + 1) = ar;         { write to output auto-buffer }
        dm (tx_buf + 2) = ar;         { write to output auto-buffer }
        rti;

nextdigit:  call newsound;
        ax0=dm(i0,m0);      { read next digit out of list }
        ay0=h#F000;
        ar=ax0 and ay0;
        if eq jump notstop;
stop:       ar=0;
        dm(state)=ar;
        i0=^dial_list;
        rti;
notstop:    ay0=h#0F00;
        ar=ax0 and ay0;
        if eq jump notredial;
redial:     i0=^dial_list;
        rti;
notredial:  ay0=h#00F0;
        ar=ax0 and ay0;
        if eq jump newdigit;
space:      ax0=dm(time_on);
        ay0=dm(time_off);
        ar=ax0+ay0;
        dm(time_off)=ar;
        ar=0;
        dm(time_on)=ar;
        rti;
newdigit:   ay0=h#000F;
        ar=ax0 and ay0;
        sr=lshift ar by 1 (hi);
        ay0=^digits;
        ar=sr1+ay0;
        i1=ar;
        ax0=dm(i1,m0);      { look up row freq for digit }
        dm(hertz1)=ax0;
        ax0=dm(i1,m0);      { look up col freq for digit }
        dm(hertz2)=ax0;
        rti;



{------------------------------------------------------------------------------}
{
    Included in DTMF source code July 3, 1990 B.W.
    Sine Approximation                Y = boot_sin(x)
    Calling Parameters
        AX0 = x in scaled 1.15 format
        M7 = 1
        L7 = 0
    Return Values           AR = y in 1.15 format
    Altered Registers       AY0,AF,AR,MY1,MX1,MF,MR,SR,I3
    Computation Time        25 cycles}
.ENTRY  boot_sin;

.VAR/PM sin_coeff[5];
.INIT   sin_coeff: H#324000, H#005300, H#AACC00, H#08B700, H#1CCE00;

boot_sin:
    I7=^sin_coeff;                      {Pointer to coeff. buffer}
    AY0=H#4000;
    AR=AX0, AF=AX0 AND AY0;             {Check 2nd or 4th quad.}
    IF NE AR=-AX0;                      {If yes, negate input}
    AY0=H#7FFF;
    AR=AR AND AY0;                      {Remove sign bit}
    MY1=AR;
    MF=AR*MY1 (RND), MX1=PM(I7,M7);     {MF = x2}
    MR=MX1*MY1 (SS), MX1=PM(I7,M7);     {MR = C1x}
    CNTR=3;
    { DO approx UNTIL CE; }
top_approx:     MR=MR+MX1*MF (SS);
approx:         MF=AR*MF (RND), MX1=PM(I7,M7);
        if not ce jump top_approx;
    MR=MR+MX1*MF (SS);
    SR=ASHIFT MR1 BY 3 (HI);
    SR=SR OR LSHIFT MR0 BY 3 (LO);      {Convert to 1.15 format}
    AR=PASS SR1;
    IF LT AR=PASS AY0;                  {Saturate if needed}
    AF=PASS AX0;
    IF LT AR=-AR;                       {Negate output if needed}
    RTS;

{------------------------------------------------------------------------------
 -
 -  transmit interrupt used for Codec initialization
 -
 ------------------------------------------------------------------------------}
next_cmd:
        ena sec_reg;
        ax0 = dm (i3, m1);          { fetch next control word and }
        dm (tx_buf) = ax0;          { place in transmit slot 0    }
        ax0 = i3;
        ay0 = ^init_cmds;
        ar = ax0 - ay0;
        if gt rti;                  { rti if more control words still waiting }
        ax0 = 0xaf00;               { else set done flag and }
        dm (tx_buf) = ax0;          { remove MCE if done initialization }
        ax0 = 0;
        dm (stat_flag) = ax0;       { reset status flag }
        rti;


{******************************************************************************
 *  A high to low transition on flag_in signifies the start bit; it also
 *  triggers IRQ1 ISR which then turn on timer if the timer is off.  This is
 *  at to most 1/3 bit period too late but we should still catch the byte.
 ******************************************************************************}
irq1isr:
        pop sts;
        ena timer;              { start timer now }
        rts;                    { note rts }

.endmod;
