
/* *********************************************************************** */ 
/*                                                                         */ 
/*      This demo program demonstrates the general use of ADSP-21xx        */ 
/*      instructions. Running this program in the simulator will           */ 
/*      provide you with a better understanding of the basic programming   */ 
/*      of the ADSP-2100 family DSPs. Programmers who have little or no    */ 
/*      experience with the ADSP-2100 Family should single-step through    */ 
/*      this program within the simulator. Inspecting register contents    */ 
/*      and flag values should help in understanding how the processor     */ 
/*      executes each instruction.                                         */ 
/*                                                                         */ 
/*      This program performs some general arithmetic calculations,        */ 
/*      stores and retrieves data from a circular buffer in data memory    */ 
/*      and toggles the flag pin within a loop. The flag pin is connected  */ 
/*      to the red LED on the EZ-KIT Lite board. If you run this program   */ 
/*      on the EZ-KIT Lite board, you should see the red LED blink.        */ 
/*                                                                         */ 
/* *********************************************************************** */ 
.module/ram/abs=0    demo;       /* define the program module */ 
.var/dm/circ   buffer[10];       /* declare a circular buffer of length 10  */ 
.var/dm        flag;             /* general purpose flag */
   
        jump start;              /* jump over interrupt vectors */ 
        nop; nop; nop;

        rti; rti; rti; rti;      /* code vectors here upon IRQ2 interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon IRQL1 interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon IRQL0 interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon SPORT0 TX interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon SPORT0 RX interrupt */ 
        
 /* code vectors here upon IRQE interrupt */        
        ax0 = 0x000f;         /* set flag to non-zero value */
        dm(flag) = ax0; 
        rti;            
        rti;      
        
        rti; rti; rti; rti;      /* code vectors here upon BDMA interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon SPORT1 TX (IRQ1) interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon SPORT1 RX (IRQ0) interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon TIMER interrupt */ 
        rti; rti; rti; rti;      /* code vectors here upon POWER DOWN interrupt */ 

start:  imask = 0x0010;       /* enable IRQE, interrupt button on EZ-LAB */
        ax0 = 0;
        dm(flag) = ax0;      /* clear flag */
        ax0 = 3;              /* load a 3 into an ALU X input register */ 
        ay1 = 5;              /* load a 5 into an ALU Y input register */ 
        ar = ax0 + ay1;       /* add inputs, store sum in result register */ 
        af = ax0 - ay1;       /* subtract, store result in feednack reg */    
                              /* look at ALU flags in simlator          */ 

        ena m_mode;           /* set up multiplier for integer arithmetic */ 
        mx0 = 2;              /* load a 2 into a MAC X input register */ 
        mx1 = 3;              /* load a 3 into another MAC X input register */ 
        my0 = 2;                /* load a 2 into a MAC Y input register */ 
        mr = mx0 * my0 (uu);     /* multiply numbers as unsigned numbers */ 
        mr = mr + mx1 * my0 (uu); /* perform multiply and accumulate */ 

/*  The next section of code uses looping and circular data buffers.       */         
/*  Experiment by using different counter values for the loops and see     */ 
/*  the affect on the rate of the flashing LED. The loops are kept short   */                          
/*  to make the simulation exercise more convenient. Single-step this      */                 
/*  program in the simulator and look at the register and memory contents. */ 
/*  Also, note the data memory address values (I register) as the circular */ 
/*  buffer is automatically maintained.                                    */ 

        i0 = ^buffer;    /* load address pointer with start address of buffer */ 
        l0 = %buffer;    /* load length register with length og buffer */ 
        m0 = 1;          /* load address modify register with 1 to increment */ 
        m1 = 3;          /* load second modifier with a 3 */ 

again:  cntr = 3;            /* load loop counter */
        do loop_1 until ce;  /* set up loop to end when count expires */ 
          dm(i0, m0) = ar;   /* store contents of register ar in buffer */ 
          cntr = 2;          /* load loop counter, cntr stack allows nesting */ 
          do loop_2 until ce;      /* repeat to loop_2, 2 times */ 
            cntr = 10;       /* load loop counter with 10, change to 0x1000 to run on EZ-LAB */ 
            do loop_3 until ce;    /* repeat from her to loop_3, 10 times */ 
              ar = ar + ay1;       /* add contents of ay1 to ar */  
loop_3:       dm(i0, m0) = ar;
loop_2:     dm(i0, m1) = mr1;
loop_1: mr1 = ar;                   /* move contents of ar into mr1 */ 
        toggle fl1;             /* toggle flag */ 
        ax0 = dm(flag);         /* get flag value from memory */
        none = pass ax0;        /* generate ALU arithmetic flag */
        if ne rts;              /* if not 0, return to monitor */
        jump again;             /*  go back to start of nesed loops  */ 

.endmod;


