 /*******************************************************************************
 *              EZ-KIT LITE TALKTHRU DEMO       11/28/95  
 *
 *  This demo program performs a talkthru, loopback function. The DSP reads the stereo 
 *  ouput of the AD1847 and immediately writes the stereo data back to the AD1847.
 *  Pressing the IRQE interrupt button on the ADSP-2181 demo board will 
 *  toggle the flag-out LED ( FL1 ).
 *
 *  This sample program is organized into the following sections:
 *      Assemble time constants
 *      Interrupt vector table
 *      ADSP 2181 initialization
 *      ADSP 1847 Codec initialization
 *      Interrupt service routines                                             */
 
.module/RAM/ABS=0 loopback;


 /*==================  ASSEMBLE TIME CONSTANTS ==========================*/

.const  IDMA=            0x3fe0;
.const  BDMA_BIAD=       0x3fe1;
.const  BDMA_BEAD=       0x3fe2;
.const  BDMA_BDMA_Ctrl=  0x3fe3;
.const  BDMA_BWCOUNT=    0x3fe4;
.const  PFDATA=          0x3fe5;
.const  PFTYPE=          0x3fe6;
.const  SPORT1_Autobuf=  0x3fef;
.const  SPORT1_RFSDIV=   0x3ff0;
.const  SPORT1_SCLKDIV=  0x3ff1;
.const  SPORT1_Control_Reg= 0x3ff2;
.const  SPORT0_Autobuf=  0x3ff3;
.const  SPORT0_RFSDIV=   0x3ff4;
.const  SPORT0_SCLKDIV=  0x3ff5;
.const  SPORT0_Control_Reg=   0x3ff6;
.const  SPORT0_TX_Channels0=  0x3ff7;
.const  SPORT0_TX_Channels1=  0x3ff8;
.const  SPORT0_RX_Channels0=  0x3ff9;
.const  SPORT0_RX_Channels1=  0x3ffa;
.const  TSCALE=  0x3ffb;
.const  TCOUNT=  0x3ffc;
.const  TPERIOD= 0x3ffd;
.const  DM_Wait_Reg= 0x3ffe;
.const  System_Control_Reg= 0x3fff;


/*===============  DATA BUFFER DECLARATIONS  =============================*/

.var/dm/ram/circ        rx_buf[3];       /* declare receive buffer, Status + L data + R data    */
.var/dm/ram/circ        tx_buf[3];       /* declare transmit buffer, Cmd + L data + R data      */
.var/dm/ram/circ        init_cmds[13];   /* declare buffer with AD1847 initialization codes     */
.var/dm                 stat_flag;       /* declare a variable to be used as a status flag      */



/*================ DATA BUFFER INITIALIZATIONS ===========================*/


/* The AD1847 CODEC communicates with the DSP serially. The AD1847 transmits and receives three 16 bit 
 * words per sample. In this program we declare two, 3 location circular buffers, tx_buf for the transmit 
 * data buffer and rx_buf for the receive data buffer. The AD1847 receives from the DSP's tx_buf, a 
 * command word, left channel data, and right channel data each sample. The AD1847 transmits to the DSP 
 * rx_buf a status word, left channel data and right channel data each sample. The DSP's serial port 
 * communicating with the AD1847 is configured to use autobuffering and multi-channel mode. This allows 
 * the DSP's serial port to automatically access memory and automatically transmit/receive three words at 
 * a time. Once the DSP's serial port is configured and running, it will run forever interrupting the DSP 
 * when new data has been received. This means the DSP reads the AD1847's data by reading the rx_buf buffer 
 * and writes to the AD1847 by writing to the tx_buf buffer. The DSP knows when it can read/write the 
 * AD1847 via the serial port interrupts.                */

.init tx_buf:   0xc000, 0x0000, 0x0000;         /* Initialize the tx_buff with these values. 
						 * Command word = 0xc0000 
						 * Left channel data = 0x0000 
						 * Right channel data = 0x0000.
						 * This command word will place AD1847 in command   
						 * mode by setting the MCE bit                                   */

/* Initialize the init_cmds buffer with the following values. These are the initial configuration values for    
 * the AD1847 CODEC. Bits 0-7 are the 8 bits of data for  the index registers ( data0-7 ). Bits 8-11 are the 
 * address for the index registers ( IA0-3 ). Bit 13 sets the part into readback  mode ( RREQ ). Bit 14 is the 
 * Mode Change Enable bit ( MCE ). This must be set when changing the index registers. Bit 15 is the Clear           
 * Overrange bit ( CLOR ). This bit determines the overrange status mode. The program is set up to automatically               
 * load this buffer into the AD1847.                                                                            */

.init init_cmds:
	0xc002,     /*  CLOR set, MCE set, Index reg address=0, data=0x2
		     *  Left input control reg
		     *  b7-6: 0=left line 1
		     *        1=left aux 1
		     *        2=left line 2
		     *        3=left line 1 post-mixed loopback
		     *  b5-4: res
		     *  b3-0: left input gain x 1.5 dB               */
								 
	0xc102,     /*  CLOR set, MCE set, Index reg address=1, data=0x2
		     *  Right input control reg
		     *  b7-6:  0=right line 1
		     *         1=right aux 1
		     *         2=right line 2
		     *         3=right line 1 post-mixed loopback
		     *  b5-4:  res
		     *  b3-0:  right input gain x 1.5 db               */

	0xc288,     /*  CLOR set, MCE set, Index reg address=2, data=0x88
		     *   Left aux 1 control reg
		     *   b7  :    1=left aux 1 mute
		     *   b6-5:    res
		     *   b4-0:    gain/atten x 1.5, 08= 0dB, 00= 12dB    */
		    
	0xc388,     /* CLOR set, MCE set, Index reg address=3, data=0x88
		     *   Right aux 1 control reg
		     *   b7  :   1=right aux 1 mute
		     *   b6-5:   res
		     *   b4-0:   gain/atten x 1.5, 08= 0dB, 00= 12dB     */
		    
	0xc488,     /* CLOR set, MCE set, Index reg address=4, data=0x88
		     *   left aux 2 control reg
		     *   b7  :   1=left aux 2 mute
		     *   b6-5:   res
		     *   b4-0:   gain/atten x 1.5, 08= 0dB, 00= 12dB     */
		    
	0xc588,     /* CLOR set, MCE set, Index reg address=5, data=0x88
		     *   right aux 2 control reg
		     *   b7  :   1=right aux 2 mute
		     *   b6-5:   res
		     *   b4-0:   gain/atten x 1.5, 08= 0dB, 00= 12dB     */
		    
	0xc680,     /* CLOR set, MCE set, Index reg address=6, data=0x80
		     *   left DAC control reg
		     *   b7  :   1=left DAC mute
		     *   b6  :   res
		     *   b5-0:   attenuation x 1.5 dB    */

	0xc780,     /* CLOR set, MCE set, Index reg address=7, data=0x80
		     *   right DAC control reg
		     *   b7  :   1 = right DAC mute
		     *   b6  :   res
		     *   b5-0:   attenuation x 1.5 dB    */
		    
	0xc850,     /* CLOR set, MCE set, Index reg address=8, data=0x5c
		     *   data format register
		     *   b7  :   res
		     *   b5-6:   0 = 8-bit unsigned linear PCM
		     *           1 = 8-bit u-law companded
		     *           2 = 16-bit signed linear PCM
		     *           3 = 8-bit A-law companded
		     *   b4  :   0 = mono, 1 =  stereo
		     *   b0-3:   0 =   8.00000 Khz
		     *           1 =   5.51250 Khz
		     *           2 = 16.00000 Khz
		     *           3 = 11.02500 Khz
		     *           4 = 27.42857 Khz
		     *           5 = 18.90000 Khz
		     *           6 = 32.00000 Khz
		     *           7 = 22.05000 Khz
		     *           8 =   .
		     *           9 = 37.80000 Khz
		     *           a =   .
		     *           b = 44.10000 Khz
		     *           c = 48.00000 Khz
		     *           d = 33.07500 Khz
		     *           e =   9.60000 Khz
		     *           f =   6.61500 Khz
		     *  (b0) :   0 = XTAL1 24.576 Mhz; 1 = XTAL2 16.9344 Mhz     */
		    
	0xc909,     /* CLOR set, MCE set, Index reg address=9, data=0x09
		     *   interface configuration reg
		     *   b7-4:   res
		     *   b3  :   1 = autocalibrate
		     *   b2-1:   res
		     *   b0  :   1 = playback enabled    */
		     
	0xca00,     /* CLOR set, MCE set, Index reg address=0xa, data=0
		     *   pin control reg
		     *   b7  :   logic state of pin XCTL1
		     *   b6  :   logic state of pin XCTL0
		     *   b5  :   master - 1 = tri-state CLKOUT
		     *           slave  - x = tri-state CLKOUT
		     *   b4-0:   res                             */
		    

	0xcc40,     /* CLOR set, MCE set, Index reg address=0xc, data=0x40
		     *   miscellaneous information reg
		     *   b7  :   1 = 16 slots per frame
		     *           0 = 32 slots per frame
		     *   b6  :   1 = 2-wire system
		     *           0 = 1-wire system
		     *   b5-0:   res                     */

		    
	0xcd00;     /* CLOR set, MCE set, Index reg address=0xd, data=0
		     *   digital mix control reg
		     *   b7-2:   attenuation x 1.5 dB
		     *   b1  :   res
		     *   b0  :   1 = digital mix enabled */

/*================== INTERRUPT VECTOR TABLE ==============================*/

/*   Each interrupt vector has 4 PM memory locations. When an interrupt occurs the
 *   DSP will jump to the location of the interrupt vector providing the interrupt 
 *   is unmasked.                                                                */

	jump start;               /* address = 0x00: reset interrupt vector                */
	 rti; 
	 rti; 
	 rti;

	rti;                      /* address = 0x04: IRQ2 interrupt vector                 */
	 rti; 
	 rti; 
	 rti;

	rti;                      /* address = 0x08: IRQL1 interrupt vector                */
	 rti;
	 rti;
	 rti;

	rti;                      /* address = 0x0c: IRQL0 interrupt vector                */
	 rti; 
	 rti; 
	 rti;

	ar = dm(stat_flag);       /* address = 0x10: SPORT0 tx interrupt vector            */
		ar = pass ar;
		if eq rti;
	 jump next_cmd;

	jump input_samples;       /* address = 0x14: SPORT1 rx interrupt vector            */
	 rti;
	 rti;
	 rti;

	jump irqe;                /* address = 0x18: IRQE interrupt vector                 */
	 rti;                     /* This is the interrupt vector used by the interrupt    */
	 rti;                     /* switch on the EZ-KIT LITE board                       */    
	 rti;

	rti;                      /* address = 0x1c: BDMA interrupt vector                 */
	 rti; 
	 rti; 
	 rti;

	rti;                      /* address = 0x20: SPORT1 tx or IRQ1 interrupt vector    */
	 rti;
	 rti; 
	 rti;

	rti;                      /* address = 0x24: SPORT1 rx or IRQ0 interrupt vector    */
	 rti;
	 rti; 
	 rti;

	rti;                      /* address = 0x28: timer interrupt vector                */
	 rti; 
	 rti; 
	 rti;

	rti;                      /* address = 0x2c: power down interrupt vector           */
	 rti; 
	 rti; 
	 rti;

/*====================  ADSP 2181 INITIALIZATION =========================*/

/*---------------- Data Address Generator Initialization  ----------------*/

start:
	i0 = ^rx_buf;           /* set i0 = the starting address of  rx_buf     */
	l0 = %rx_buf;           /* set l0 = the length of rx_buf                */
	i1 = ^tx_buf;           /* set i1 = the starting address of  tx_buf     */
	l1 = %tx_buf;           /* set l1 = the length of tx_buf                */
	i3 = ^init_cmds;        /* set i3 = the starting address of  init_cmds  */
	l3 = %init_cmds;        /* set l3 = the length of  init_cmds            */
	l5 = 0;                 /* set l5=0 because fb_gain is not a circular   */
	m1 = 1;                 /* set modify values. Any modify value can be   */    
	m0 = 0;                 /* used with any i register as long as they are */


/*-------------------------- S E R I A L   P O R T   #0   S T U F F ---------------------------------------*/

/* This code configures the serial ports on the DSP. The serial ports are configured by writing 
 * to memory mapped control registers. SPORT0 is configured to use autobuffering 
 * and multi-channel mode with an external serial clock and external frame sync.
 * Autobuffering can be thought of as serial port DMA. The serial port uses the address
 * generators to access data memory automatically. In this program autobuffering is configured 
 * to use i0, i1, and m1. These are data address generator registers. i0 points to the rx_buf buffer, 
 * i1 points to the tx_buf buffer and m1 = 1. 
 * Multi-channel mode is enabled and configured for 32 channels. The AD1847 is also configured for 32
 * channels. In Multi-channel mode each channel is a word and associated with a time slot. In each of 
 * the 32 time slot the serial port can transmit and/or receive data or do nothing, tristating the 
 * data for that time slot. One frame consists of all 32 channels. One frame sync will start a frame 
 * transfer ( all 32 words ). This program configures the serial port 0 to transmit on channels 
 * 0, 1, 2, 16, 17 and 18 for transmit and channels 0, 1, 2, 16, 17, and 18 for receive. It 
 * will ignore all other time slots.   
 * External serial clock and external frame-syncs mean they are inputs to the DSP.        */

	ax0 =0x287;   
	dm (SPORT0_Autobuf) = ax0;      /*  enable receive and transmit autobuffering,
					 *  RMREG=m1, RIREG=i0, TMREG=m1, TIREG=i1,
					 *  CLKOUT enabled, BIASRND disabled           */

	ax0 = 0;         
	dm (SPORT0_RFSDIV) = ax0;       /*  RFSDIV = SCLK Hz/RFS Hz - 1 
					 *  using external      receive frame sync ( RFS=input )        */

	ax0 = 0;         
	dm (SPORT0_SCLKDIV) = ax0;      /*  SCLK = CLKOUT / (2  (SCLKDIV + 1) 
					 *  using external serial clock ( SCLK=input )        */

	ax0 =0x860f;   
	dm (SPORT0_Control_Reg) = ax0;  /*  SLEN= 16 bits, right justify, zero-fill unused MSBs,
					 *  INVRFS=0, INVTFS=0, IRFS=0, ITFS=1, 
					 *  MFD=1, ISCLK=0, MCE=1                       */

	ax0 =0x7;      
       dm (SPORT0_TX_Channels0) = ax0;  /* enable channels 0, 1 and 2 for transmit      */

	ax0 =0x7;      
	dm (SPORT0_TX_Channels1) = ax0; /* enable channels 16, 17 and 18 for transmit   */

	ax0 =0x7;      
	dm (SPORT0_RX_Channels0) = ax0; /* enable channels 0, 1 and 2 for receive        */

	ax0 =0x7;      
	dm (SPORT0_RX_Channels1) = ax0; /* enable channels 16, 17 and 18 for receive      */

	  
/*------------------ S Y S T E M   A N D   M E M O R Y   S T U F F ------------------------*/

	ax0 = 0xfff;   
	dm (DM_Wait_Reg) = ax0;         /*  Set all IOWAIT ranges to 7 wait states, 
					 *  set DWAIT to 0 wait states                          */

	ax0 = 0x1000;                   /* PWAIT = 0, enable SPORT0                             */
	dm (System_Control_Reg) = ax0;

	ifc = 0xff;                     /* clear pending interrupt                              */
	nop;
	icntl = 0;                      /* external interrupts set to level sensitivity, disable
					 * nested interrupts                                    */   
	mstat = 0x40;                   /* enable go mode                                       */


/*==========================  AD1847 CODEC INITIALIZATION ============================================*/

	ax0 = 1;
	dm(stat_flag) = ax0;            /* initialize stat_flag to 1                                    */
	imask = b#0001000000;           /* unmask SPORT0s transmit interrupt                            */
	ax0 = dm (i1, m1);              /* set ax0 = the first value of the tx_buf                      */
	tx0 = ax0;                      /* begin autobuffer transmit by writing first value to tx0 reg
					 * (when autobuffer transmit completes a transmit interrupt 
					 * will be generated)                                           */

/*  Test for entire init_cmds  buffer to be sent to the codec   */

check_init:
	ax0 = dm (stat_flag);   /* set ax0 = the value at stat_flag. Flag set in SPORT0 tx interrupt 
				 * routine                                                              */      
	af = pass ax0;          /* pass the value of ax0 thru the ALU setting status flags              */                  
	if ne jump check_init;  /* if result of pass not equal to 0 jump check_init                     */

/*   Once initialized, wait for codec to come out of autocalibration by testing the ACI bit
 *   of the AD1847s status reg. This is done twice, first to ensure the codec is in autocalibration 
 *   mode, then to determine when autocalibration is complete.                                          */

	ay0 = 2;
check_aci1:                     /* loop to test ACI bit = 1 ( in autocalibration )      */
	ax0 = dm (rx_buf);      /* read status word of AD1847                   */              
	ar = ax0 and ay0;       /* AND 2 with status word                       */
	if eq jump check_aci1;  /* if result of AND is not = 0 leave loop       */

check_aci2:                     /* loop to test ACI bit =0 ( autocalibration complete ) */
	ax0 = dm (rx_buf);      /* wait for bit clear                           */
	ar = ax0 and ay0;
	if ne jump check_aci2;
	idle;

/* Once autocalibration is complete unmute left and right DAC channels by writing the appropriate 
 * index register                                                               */

	ay0 = 0xbf3f;                   /* unmute left DAC                      */
	ax0 = dm (init_cmds + 6);
	ar = ax0 AND ay0;
	dm (tx_buf) = ar;
	idle;                           /* wait for transmit to complete        */
	ax0 = dm (init_cmds + 7);       /* unmute right DAC                     */
	ar = ax0 AND ay0;
	dm (tx_buf) = ar;
	idle;                           /* wait for transmit to complete        */

	ifc = 0xff;                     /* clear any pending interrupt          */
	nop;
	imask = 0x30;                   /* unmask rx0 and  IRQE interrupts      */
 
 /* Wait in idle mode for an interrupt. When the program returns from the interrupt jump to idle mode. */
 
talkthru:       idle;
	jump talkthru;
 
/*========================= INTERRUPT SERVICE ROUTINES ==================================*/
 
/*------------------------- Receive Interrupt Service Routine ---------------------------*/
 
/* NOTE: Serial port data is being accessed using autobuffering. When using autobuffering the 
 * DSP uses the data address generators to automatically transfer serial port received data to 
 * data memory and transfer transmit data from data memory to the serial port. The serial 
 * port autobuffering is configured to access received data in the rx_buf buffer and transmit 
 * data in the tx_buf buffer.                                                             */

input_samples:
	ena sec_reg;                    /* use shadow register bank */
	ax0 = dm(rx_buf + 1);           /* get left channel data       */
	ay0 = dm(rx_buf + 2);           /* get right channel data    */
	dm (tx_buf + 1) = ax0;          /* send left channel data       */
	dm (tx_buf + 2) = ay0;          /* send right channel data      */
	rti;                            /* Return from interrupt, will pop the status register which consists
					 * MSTAT, ASTAT and IMASK ( returns to primary registers because of 
					 * pop of MSTAT                                                        */
	
 
/*-------------------- Transmit Interrupt Service Routine------------------------------*/
 
/* The transmit interrupt service routine is used to initializes the CODEC (AD1847). 
 * I3 points to the init_cmds buffer. The AD1847 is expecting a control word and two data words from the 
 * DSP. Here we load the next command word into the first location of the tx_buf buffer, the two data words 
 * have been initialized to zero in the beginning of the program. This routine also checks to see if the last data 
 * word has been transmitted. If the last data word has been transmitted take the AD1847 out of command 
 * mode and return from interrupt.  */

next_cmd:
	ena sec_reg;
	ax0 = dm (i3, m1);       /* get command word from init_cmds buffer        */
	dm (tx_buf) = ax0;       /* place command word in first location of tx_buf ( transmit slot 0)    */
	ax0 = i3;
	ay0 = ^init_cmds;
	ar = ax0 - ay0;          /* test for additional command words             */
	if gt rti;               /* rti if more control words still waiting */
	ax0 = 0xaf00;            /* else set done flag and remove MCE if done initialization   */
	dm (tx_buf) = ax0;              
	ax0 = 0;
	dm (stat_flag) = ax0;    /* reset status flag */
	rti;
/*--------------------- IRQE Interrupt Service Routine -----------------------------*/

/* The IRQE interrupt service routine toggles flag-out which is connected to
 * the FL1 LED on EZ-KIT LITE. It also reads the next feedback gain value from
 * the fb buffer and writes it to the fb_gain variable.                                         */

irqe:   toggle fl1;             /* Toggle flag-out 1            */

	rti;                    /* Return from interrupt, will pop the status register which consists
				 * MSTAT, ASTAT and IMASK ( returns to primary registers because of 
				 * pop of MSTAT                                                        */

.endmod;


